"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createVuePlugin = exports.vueHotReload = exports.vueComponentNormalizer = void 0;
const componentNormalizer_1 = require("./utils/componentNormalizer");
const vueHotReload_1 = require("./utils/vueHotReload");
const fs_1 = __importDefault(require("fs"));
const query_1 = require("./utils/query");
const pluginutils_1 = require("@rollup/pluginutils");
const main_1 = require("./main");
const template_1 = require("./template");
const descriptorCache_1 = require("./utils/descriptorCache");
const style_1 = require("./style");
const hmr_1 = require("./hmr");
const jsxTransform_1 = require("./jsxTransform");
// import acorn from 'acorn';
// import jsx from 'acorn-jsx';
exports.vueComponentNormalizer = '\0/vite/vueComponentNormalizer';
exports.vueHotReload = '\0/vite/vueHotReload';
// https://github.com/vitejs/vite/blob/e8c840abd2767445a5e49bab6540a66b941d7239/packages/vite/src/node/optimizer/scan.ts#L147
const scriptRE = /(<script\b(?:\s[^>]*>|>))(.*?)<\/script>/gims;
// https://github.com/vitejs/vite/blob/e8c840abd2767445a5e49bab6540a66b941d7239/packages/vite/src/node/optimizer/scan.ts#L151
const langRE = /\blang\s*=\s*(?:"([^"]+)"|'([^']+)'|([^\s'">]+))/im;
// class AstExt {
//   public acornExt = acorn.Parser.extend(jsx())
//   constructor() {
//   }
//   deepWalk(ast:any, cb:any) {
//     if (!ast) return;
//     if (typeof ast === 'object') {
//       for (const key of Object.keys(ast)) {
//         const bool = cb({ [key]: ast[key] });
//         if (bool === false) return;
//         this.deepWalk(ast[key], cb);
//       }
//     }
//     if (Array.isArray(ast)) {
//       for (const item of ast) {
//         const bool = cb(item);
//         if (bool === false) return;
//         this.deepWalk(item, cb);
//       }
//     }
//   }
//   async checkJSX(content:any) {
//     return new Promise(resolve => {
//       const ast = this.acornExt.parse(content, { sourceType: 'module', ecmaVersion: 2019 });
//       this.deepWalk(ast, (node: any) => {
//         if (['JSXElement', 'JSXFragment'].includes(node.type)) {
//           resolve(true);
//           return false;
//         }
//       });
//       resolve(false);
//     });
//   }
// }
const checkJSX = async (content) => {
    return new Promise(resolve => {
        if (content.indexOf('<script') > -1 && content.indexOf('</script>') > -1) {
            let hasJsx = false;
            content.replace(/<script.*?>([\s\S]+?)<\/script>/img, (_, js) => {
                // 判断script内是否包含jsx语法
                if (/<[^>]+>/.test(js)) {
                    hasJsx = true;
                }
                return js;
            });
            resolve(hasJsx);
            return false;
        }
        else if (/<[^>]+>/.test(content)) {
            resolve(true);
            return false;
        }
        resolve(false);
    });
};
function createVuePlugin(rawOptions = {}) {
    const options = {
        isProduction: process.env.NODE_ENV === 'production',
        ...rawOptions,
        root: process.cwd(),
    };
    const filter = (0, pluginutils_1.createFilter)(options.include || /\.vue$/, options.exclude);
    const name = 'vite-plugin-vue2-jsx';
    // const astExt = new AstExt;
    return {
        name,
        config(config) {
            if (!config.optimizeDeps)
                config.optimizeDeps = {};
            if (!config.optimizeDeps.esbuildOptions)
                config.optimizeDeps.esbuildOptions = {};
            if (!config.optimizeDeps.esbuildOptions.plugins)
                config.optimizeDeps.esbuildOptions.plugins = [];
            config.optimizeDeps.esbuildOptions.plugins.push({
                name,
                async setup(build) {
                    build.onLoad({ filter: /\.vue$/ }, async ({ path }) => {
                        const raw = fs_1.default.readFileSync(path, 'utf8');
                        let js = '';
                        let loader = 'js';
                        let match = null;
                        scriptRE.lastIndex = 0;
                        // https://github.com/vitejs/vite/blob/e8c840abd2767445a5e49bab6540a66b941d7239/packages/vite/src/node/optimizer/scan.ts#L240
                        while (match = scriptRE.exec(raw)) {
                            const [, openTag, content] = match;
                            const langMatch = openTag.match(langRE);
                            const lang = langMatch && (langMatch[1] || langMatch[2] || langMatch[3]);
                            if (lang === 'ts' || lang === 'tsx' || lang === 'jsx') {
                                loader = lang;
                            }
                            else if (await checkJSX(content)) {
                                loader = 'jsx';
                            }
                            js = content;
                        }
                        return {
                            loader,
                            contents: js,
                        };
                    });
                    build.onLoad({ filter: /\.js$/ }, async ({ path }) => {
                        const raw = fs_1.default.readFileSync(path, 'utf8');
                        if (/<[^>]+>/.test(raw)) {
                            return {
                                loader: 'jsx',
                                contents: raw,
                            };
                        }
                    });
                },
            });
            if (options.jsx) {
                return {
                    esbuild: {
                        include: /\.ts$/,
                        exclude: /\.(tsx|jsx)$/,
                    },
                };
            }
        },
        handleHotUpdate(ctx) {
            if (!filter(ctx.file)) {
                return;
            }
            return (0, hmr_1.handleHotUpdate)(ctx, options);
        },
        configResolved(config) {
            options.isProduction = config.isProduction;
            options.root = config.root;
        },
        configureServer(server) {
            options.devServer = server;
        },
        // 处理 ES6 的 import 语句，最后需要返回一个模块的 id
        async resolveId(id) {
            if (id === exports.vueComponentNormalizer || id === exports.vueHotReload) {
                return id;
            }
            // serve subpart requests (*?vue) as virtual modules
            if ((0, query_1.parseVueRequest)(id).query.vue) {
                return id;
            }
        },
        // 执行每个 rollup plugin 的 load 方法，产出 ast 数据等
        load(id) {
            if (id === exports.vueComponentNormalizer) {
                return componentNormalizer_1.normalizeComponentCode;
            }
            if (id === exports.vueHotReload) {
                return vueHotReload_1.vueHotReloadCode;
            }
            const { filename, query } = (0, query_1.parseVueRequest)(id);
            // select corresponding block for subpart virtual modules
            if (query.vue) {
                if (query.src) {
                    return fs_1.default.readFileSync(filename, 'utf-8');
                }
                const descriptor = (0, descriptorCache_1.getDescriptor)(filename);
                let block;
                if (query.type === 'script') {
                    block = descriptor.script;
                }
                else if (query.type === 'template') {
                    block = descriptor.template;
                }
                else if (query.type === 'style') {
                    block = descriptor.styles[query.index];
                }
                else if (query.index != null) {
                    block = descriptor.customBlocks[query.index];
                }
                if (block) {
                    return {
                        code: block.content,
                        map: block.map,
                    };
                }
            }
        },
        async transform(fileCode, id, transformOptions) {
            let code = fileCode;
            const { filename, query } = (0, query_1.parseVueRequest)(id);
            if (/\.(vue)$/.test(id)) {
                let hasJsx = false;
                fileCode.replace(/<script.*?>([\s\S]+?)<\/script>/img, (_, js) => {
                    // 判断script内是否包含jsx语法和是否已加lang="jsx"
                    if (/<[^>]+>/.test(js) &&
                        /<script.*?>/.test(_) &&
                        !(/<script\s*lang=("|')jsx("|').*?>/.test(_))) {
                        hasJsx = true;
                    }
                    return js;
                });
                if (hasJsx) {
                    code = fileCode.replace('<script', '<script lang="jsx"');
                }
            }
            if (/\.(tsx|jsx)$/.test(id)) {
                return (0, jsxTransform_1.transformVueJsx)(code, id, options.jsxOptions);
            }
            // js文件包含jsx语法自动转换
            if (!query.vue && /\.(js)$/.test(id) && /<[^>]+>/.test(code)) {
                return (0, jsxTransform_1.transformVueJsx)(code, id, options.jsxOptions);
            }
            if ((!query.vue && !filter(filename)) || query.raw) {
                return;
            }
            if (!query.vue) {
                // main request
                return await (0, main_1.transformMain)(code, filename, options, this);
            }
            const descriptor = (0, descriptorCache_1.getDescriptor)(query.from ? decodeURIComponent(query.from) : filename);
            // sub block request
            if (query.type === 'template') {
                return (0, template_1.compileSFCTemplate)(code, descriptor.template, filename, options, this);
            }
            if (query.type === 'style') {
                return await (0, style_1.transformStyle)(code, filename, descriptor, Number(query.index), this);
            }
        },
    };
}
exports.createVuePlugin = createVuePlugin;
//# sourceMappingURL=index.js.map